/*	Bluetooth_PTT 
	v1, December 2009

	By bertrand Zauhar, VE2ZAZ
	ve2zaz@amsat.org
	http://ve2zaz.net

	TARGET device: PIC12F683
	Compiler: SourceBoost's BoostC for PIC
	IDE: SourceBoost IDE

	Note: The watchdog timer is used to reset the chip (thus release the PTT) when PTT it is active for more than 2.25 minutes.
*/

#include <PIC12F683.h>  // Probably redundant
#include <system.h>
#include <string.h>
// #include <icd2.h>   // Only needed if real time debugging using ICD2 is required

// GLOBAL VARIABLES
unsigned char 		delay_ctr 				= 0; 
// none

// CONFIGURATION BYTES
#pragma DATA _CONFIG, _FCMEN_OFF & _IESO_OFF & _BOD_OFF & _CPD_OFF & _CP_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDT_OFF & _INTOSCIO  
#pragma CLOCK_FREQ 31000		// Tells the compiler what clock frequency is used. Important for delay functions

// CONSTANTS AND MASKS DEFINITION

#define btooth_sw_in			1 	//The bit position of the GPIO port corresponding to the bluetooth pushbutton input.
#define ptt_out					5 	//The bit position of the GPIO port corresponding to the PTT output.
#define btooth_comp_out			6	//The bit position in register CMCON0 that provides comparatour output state.
#define swdten					0	//the bit position in WDTCON that enables the WDT

// Interrupt Service Routine
/* 
void interrupt( void )
{
}	
*/
// End of interrupt routine


// EXECUTED CODE STARTS HERE
void main(void)
{
	// Initialization stage
	intcon 		= 	0b01001000; 	// disable interrupts, enable peripheral ints, enable GPIO change ints.	
	ioc    		=	0b00000000;		// disable gpio interrupts on change
	pie1   		=	0b00000000; 	// disable various interrupts
	option_reg 	= 	0b00001111;		// weak pullups global enable, prescaler assigned to WDT and set to 1:128
	wpu    		=	0b00110111;		// individual weak pullups enabled
	osccon		=	0b00000001; 	// select internal oscillator and set it to 31Khz	
	t1con		=	0b00110000;		// disable timer1 and set prescaler div. by 8 
	t2con		=	0b01111010;		// disable timer2 and set prescaler div. by 16 and postscaler div. by 16 
	adcon0		=	0b00000000;		// disable A/D converter
	ansel		=	0b00000010;		// Enable analog assignment to pin CIN- (GP1)
	cmcon0		=	0b00010100;		// enable comparator, use internal reference, invert output.
	cmcon1 		=	0b00000000;		// comparator gating timer 1 asynchronously
	vrcon 		=	0b10100001;		// Enable internal comparator Vref, use low range, set Vref to 0.15V 
	wdtcon		=	0b00010100;		// Assign the right prescaler value to the watchdog timer. Keep WTD off for now.
	gpio 		=	0b00000000;		// Disable PTT (low), lows on other Port pins.
	trisio 		=	0b00011111;		// GPIO first disabled, but assigned as follows:    
									  //	5-PTT output (active high, passed through a NPN transistor and also feeds LED)                                
									  //	4-Available
									  //	3-Not available. Assigned to MCLR input.
									  //	2-Available 
									  //	1-Bluetooth pushbutton input from Jabra A210
									  //	0-Available
	delay_ms(250);					// Generates a startup delay of 0.5 second
	delay_ms(250);					// 
	set_bit(wdtcon,swdten);			// Enable WDT

	// Actual Bluetooth pushbutton analysis code starts here
	while(1)
	{	
		while (!test_bit(cmcon0,btooth_comp_out) && (delay_ctr < 100)) // Debouncing. Loop for up to 100ms while the A210 sends a solid short to ground
		{
			delay_ms(1);
			delay_ctr = delay_ctr + 1;
		}
		if (delay_ctr == 100) 	// Debouncing completed on comparator output?
		{
			if (!test_bit(gpio,ptt_out))   //Case where PTT is idle
			{
				set_bit(gpio,ptt_out);		// Engage PTT
			}
			else 				//Case where PTT is engaged
			{
				clear_bit(gpio,ptt_out);	// Release PTT
			}
		while(!test_bit(cmcon0,btooth_comp_out));	//wait until button is released before proceeding
		}
		delay_ctr = 0;
		if (!test_bit(gpio,ptt_out)) //If the PTT is not engaged, clear the watchdog timer.
		{
			clear_wdt();	// BoostC-defined function
		}
	}
}

